
import tbplots

if __name__ == "__main__":
    iFilter=["LR1e5","LR5e5","LR1e4"]
    tbplots.PlotTensorflowData(path=".",gFilter="4R_PPO",iFilter=iFilter,
        metric="Env Results/WinRate",saveName="LR_HP_Median",
        title="4 Rooms PPO Learning Rate Exp",
        xlabel="Episode",ylabel="Win Rate"
        )
    tbplots.PlotTensorflowData(path=".",gFilter="4R_PPO",iFilter=iFilter,
        metric="Env Results/WinRate",saveName="LR_HP_Mean",
        title="4 Rooms PPO Learning Rate Exp",
        xlabel="Episode",ylabel="Win Rate",plotStyle="Confidence",
        )




import os
from os import walk
from scipy.signal import savgol_filter
import matplotlib.pyplot as plt
import numpy as np

from .math import GetAverageSTD,GetAverageConfidence,GetMedian
from .tbUtils import LoadTensorboard

DEFAULT_COLORS = ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']

def PlotTensorflowData(path,gFilter,iFilter,metric,saveName,labels=None,
    title=None,xlabel=None,ylabel=None,xbounds=None,ybounds=None,plotStyle="Median",
    showReplicates=False,smoothing=False,smoothingWindow=11,savePath=""):
    """ Function Description: TBC"""

    assert(len(gFilter)<len(DEFAULT_COLORS), "Recieved >10 independent experiments. Reduce number to be plotted.")

    #Metric can be string or list. If input is a list, then the
    if isinstance(metric,str):
        metric=[metric]*len(iFilter)

    #If no Labels are provided, defaults to using
    if labels is None:
        labels = iFilter
    else:
        assert(len(labels)==len(iFilter), "The length of labels is different from the specified iFilter")

    print("Plotting Tensorboard Data with the following configs: \n\tgFilter:{} | filePath:{} \n\tiFilters:{}".format(gFilter,path,iFilter))
    #Iterating through the specified directory looking for matches to the specified characteristics.
    dataFiles = {}
    for name,label in zip(iFilter,labels):
        dataFiles[label]=[]
        for (dirpath, dirnames, filenames) in walk(path):
            if len(filenames) != 0:
                if gFilter in dirpath:
                    if name in dirpath:
                        for filename in filenames:
                            if "events.out.tfevents" in filename:
                                dataFiles[label].append(LoadTensorboard("{}/{}".format(dirpath,filename)))
                                print("\tLoaded data from: {}/{}".format(dirpath,filename))
                                continue

    i=0
    fig = plt.figure(figsize=(6.5, 4))
    for label,data in dataFiles.items():

        if plotStyle in ["Average","Mean"]:
            ave,std,x,replicates =GetAverageSTD(data,metric[i])
            if smoothing:
                if len(ave)>smoothingWindow:
                    std = savgol_filter(std, smoothingWindow, 3)
                    ave = savgol_filter(ave, smoothingWindow, 3)
            if showReplicates:
                label += " ({})".format(replicates)
            plt.plot(x,ave,label=label,color=DEFAULT_COLORS[i])
            plt.fill_between(x, ave-std, ave+std,alpha=0.05,color=DEFAULT_COLORS[i])
            plt.plot(x,ave-std,alpha=0.25,color=DEFAULT_COLORS[i],linestyle=':')
            plt.plot(x,ave+std,alpha=0.25,color=DEFAULT_COLORS[i],linestyle=':')

        if plotStyle in ["Confidence"]:
            ave,conf,x,replicates =GetAverageConfidence(data,metric[i])
            if smoothing:
                if len(ave)>smoothingWindow:
                    conf = savgol_filter(conf, smoothingWindow, 3)
                    ave = savgol_filter(ave, smoothingWindow, 3)
            if showReplicates:
                label += " ({})".format(replicates)
            plt.plot(x,ave,label=label,color=DEFAULT_COLORS[i])
            plt.fill_between(x, ave-conf, ave+conf,alpha=0.05,color=DEFAULT_COLORS[i])
            plt.plot(x,ave-conf,alpha=0.25,color=DEFAULT_COLORS[i],linestyle=':')
            plt.plot(x,ave+conf,alpha=0.25,color=DEFAULT_COLORS[i],linestyle=':')

        elif plotStyle == "Median":
            median, upperQuart, lowerQuart, x,replicates  = GetMedian(data,metric[i])
            bounds=np.stack([median-lowerQuart,upperQuart-median])
            if smoothing:
                if len(median)>smoothingWindow:
                    median = savgol_filter(median, smoothingWindow, 3)
                    upperQuart = savgol_filter(upperQuart, smoothingWindow, 3)
                    lowerQuart = savgol_filter(lowerQuart, smoothingWindow, 3)

            if showReplicates:
                label += " ({})".format(replicates)
            plt.plot(x,median,label=label,color=DEFAULT_COLORS[i])
            plt.fill_between(x, lowerQuart, upperQuart,alpha=0.05,color=DEFAULT_COLORS[i])
            plt.plot(x,lowerQuart,alpha=0.25,color=DEFAULT_COLORS[i],linestyle=':')
            plt.plot(x,upperQuart,alpha=0.25,color=DEFAULT_COLORS[i],linestyle=':')

        i+=1

    plt.xlim(xbounds)
    plt.ylim(ybounds)
    plt.title(title)
    plt.ylabel(ylabel)
    plt.xlabel(xlabel)
    plt.legend()
    plt.savefig(os.path.join(savePath,saveName+".png"),bbox_inches='tight')
    plt.clf()




import tensorboard as tb
from tensorboard.backend.event_processing.event_accumulator import EventAccumulator
from tensorboard.util import tensor_util

def LoadTensorboard(logdir):
    event_acc = EventAccumulator(logdir,size_guidance={"scalars":0,"tensors": 0})
    event_acc.Reload()
    dataDict={}
    for key in event_acc.scalars.Keys():
        time, step, val = zip(*event_acc.Scalars(key))
        dataDict[key] = {"data":list(val),"step":list(step),"time":list(time)}
    for key in event_acc.tensors.Keys():
        time, step, val = zip(*event_acc.Tensors(key))
        #Implementation Note: Tensorflow 2.0 logging only saves values as Tensors, even if they are scalars.
        # Tensors by default use encoded format, which requires use of a decoder.
        val = TensorToScalar(val)
        dataDict[key] = {"data":list(val),"step":list(step),"time":list(time)}
    return dataDict

def TensorToScalar(array):
    try:
        x = [tensor_util.make_ndarray(val).item() for val in array]
    except:
        raise TypeError("Only Scalar Values can be processed. The Array loaded from tensorboard does not have `shape = ()`")
    return x


if __name__=="__main__":
    dict = LoadTensorboard("../test_logs/events.out.tfevents.1649876321.nvs-main.5579.5.v2")
    print(dict)



from setuptools import setup

setup(
    name='tbplots',
    version='0.1.0',
    description='A Python Package used to load data from Tensorboard Logs and Plot them.',
    url='https://github.com/Tran-Research-Group/TB-Plots',
    author='Stephen Hudson',
    author_email='shudson@anl.gov',
    license='BSD 2-clause',
    packages=['tbplots'],
    install_requires=['tensorboard',
                      'numpy',
                      'scipy',
                      'matplotlib',
                      ],

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)




import numpy as np

def ProcessDicts(dicts,metric):
    finalDictList = []
    for dict in dicts:
        if metric in dict:
            finalDictList.append(dict)
    return finalDictList

def GetAverageSTD(dicts,metric):
    dictsFinal = ProcessDicts(dicts,metric)
    length=0
    #Finding the longest array
    for dict in dictsFinal:
        if len(dict[metric]["data"])>length:
            length = len(dict[metric]["data"])
    data = []
    x = []
    for dict in dictsFinal:
        if len(dict[metric]["data"])==length:
            data.append(dict[metric]["data"])
            x.append(dict[metric]["step"])
        else:
            for i in range(length-len(dict[metric]["data"])):
                dict[metric]["data"].append(np.nan)
                dict[metric]["step"].append(np.nan)
            data.append(dict[metric]["data"])
            x.append(dict[metric]["step"])
    try:
        average = np.nanmean(np.stack(data),axis=0)
        x_average = np.nanmean(np.stack(x),axis=0)
        std = np.nanstd(np.stack(data),axis=0)
    except ValueError:
        average = np.asarray([0])
        std = np.asarray([0])
        x_average = np.asarray([0])
    return average,std,x_average,len(dictsFinal)


def GetAverageConfidence(dicts,metric,tval=1.0):
    dictsFinal = ProcessDicts(dicts,metric)
    length=0
    #Finding the longest array
    for dict in dictsFinal:
        if len(dict[metric]["data"])>length:
            length = len(dict[metric]["data"])
    data = []
    x = []
    for dict in dictsFinal:
        if len(dict[metric]["data"])==length:
            data.append(dict[metric]["data"])
            x.append(dict[metric]["step"])
        else:
            for i in range(length-len(dict[metric]["data"])):
                dict[metric]["data"].append(np.nan)
                dict[metric]["step"].append(np.nan)
            data.append(dict[metric]["data"])
            x.append(dict[metric]["step"])
    try:
        average = np.nanmean(np.stack(data),axis=0)
        x_average = np.nanmean(np.stack(x),axis=0)
        confidence = tval*np.nanstd(np.stack(data),axis=0)/np.sqrt(len(dictsFinal))
    except ValueError:
        average = np.asarray([0])
        confidence = np.asarray([0])
        x_average = np.asarray([0])
    return average,confidence,x_average,len(dictsFinal)


def GetMedian(dicts,metric):
    dictsFinal = ProcessDicts(dicts,metric)
    length=0
    #Finding the longest array
    for dict in dictsFinal:
        if len(dict[metric]["data"])>length:
            length = len(dict[metric]["data"])
    data = []
    x = []
    for dict in dictsFinal:
        if len(dict[metric]["data"])==length:
            data.append(dict[metric]["data"])
            x.append(dict[metric]["step"])
        else:
            for i in range(length-len(dict[metric]["data"])):
                dict[metric]["data"].append(np.nan)
                dict[metric]["step"].append(np.nan)
            data.append(dict[metric]["data"])
            x.append(dict[metric]["step"])
    try:
        median = np.nanmedian(np.stack(data),axis=0)
        upperQuart = np.nanquantile(np.stack(data),0.75,axis=0)
        lowerQuart = np.nanquantile(np.stack(data),0.25,axis=0)
        x_average = np.nanmean(np.stack(x),axis=0)
    except ValueError:
        median = np.asarray([0])
        upperQuart = np.asarray([0])
        lowerQuart = np.asarray([0])
        x_average = np.asarray([0])
    return median,upperQuart,lowerQuart,x_average,len(dictsFinal)
